<?php
// $Id: operations.eval.inc,v 1.6 2009/04/07 11:54:26 jpetso Exp $
/**
 * @file
 * Transformations -
 * A framework for generic schema-based data transformation pipelines.
 *
 * Copyright 2009 by Jakob Petsovits ("jpetso", http://drupal.org/user/56020)
 */


/**
 * Implementation of [module]_operation_[class]().
 */
function transformations_operation_TfPHPEvaluate() {
  return array(
    'label' => t('Evaluate PHP code'),
    'description' => t('Executes code on the parameter(s). Not typesafe or malicious-user-proof, use at your own risk.'),
    'usesTypeOptions' => TRUE,
  );
}

/**
 * Implementation of [module]_operation_[class]_type_options().
 */
function transformations_operation_TfPHPEvaluate_type_options(&$typeOptionsInfo, $prefix) {
  $typeOptions = array_combine(range(0, 50), range(0, 50));

  $typeOptionsInfo[$prefix . 'inputCount']['label'] = t('Number of parameters');
  $typeOptionsInfo[$prefix . 'inputCount']['options'] = $typeOptions;
  $typeOptionsInfo[$prefix . 'inputCount']['default_value'] = 1;
  $typeOptionsInfo[$prefix . 'outputCount']['label'] = t('Number of outputs');
  $typeOptionsInfo[$prefix . 'outputCount']['options'] = $typeOptions;
  $typeOptionsInfo[$prefix . 'outputCount']['default_value'] = 1;
}

class TfPHPEvaluate extends TfOperation {
  private $inputCount;
  private $outputCount;

  /**
   * Overriding TfOperation::initialize().
   */
  public function initialize(Array $typeOptions = array()) {
    parent::initialize($typeOptions);

    if (!isset($typeOptions['inputCount'])) {
      throw new Exception('The inputCount type option has not been set.');
    }
    if (!isset($typeOptions['outputCount'])) {
      throw new Exception('The outputCount type option has not been set.');
    }
    $this->inputCount = $typeOptions['inputCount'];
    $this->outputCount = $typeOptions['outputCount'];
  }

  /**
   * Overriding of TfOperation::typeOptions().
   */
  public function typeOptions() {
    return array(
      'inputCount' => $this->inputCount,
      'outputCount' => $this->outputCount,
    );
  }

  /**
   * Implementation of TfOperation::inputs().
   */
  protected function inputs() {
    $inputs = array('code');
    for ($i = 0; $i < $this->inputCount; ++$i) {
      $inputs[] = 'input' . $i;
    }
    return $inputs;
  }

  /**
   * Implementation of TfOperation::inputInfo().
   */
  protected function inputInfo($inputKey, $propertyKey) {
    if ($inputKey == 'code') {
      switch ($propertyKey) {
        case 'label':
          return t('PHP code');

        case 'description':
          $description = t('The PHP code that is executed.');

          if ($this->inputCount) {
            $inputs = array();
            for ($i = 0; $i < $this->inputCount; ++$i) {
              $inputs[] = '$input[' . $i . ']';
            }
            $description .= "\n\n" . t('Available input values are: !inputs. Those variables are TfDataWrapper objects - if they wrap directly accessible data then you can access that data by calling $input[$i]->data() (which works for strings, numbers, etc.), whereas if the wrapper represents a list then you can traverse it with a simple foreach loop on $input[$i].', array('!inputs' => implode(', ', $inputs))) . "\n";
          }
          if ($this->outputCount) {
            $outputs = array();
            for ($i = 0; $i < $this->outputCount; ++$i) {
              $outputs[] = '$output[' . $i . ']';
            }
            $description .= "\n" . t('All output values (!outputs) must be assigned a non-NULL value, otherwise the operation will fail with an error.', array('!outputs' => implode(', ', $outputs)));
          }

          return trim($description);

        case 'expectedSchema':
          $schema = new TfWritableSchemaElement('code', 'php:type:string');
          $schema->setFormatHint('string:multiline', TRUE);
          return $schema;
      }
    }
    switch ($propertyKey) {
      case 'label':
        return t('$input[!num]', array('!num' => substr($inputKey, 5)));

      case 'expectedSchema':
        return new TfWritableSchemaElement($inputKey);
    }
  }

  /**
   * Implementation of TfOperation::outputs().
   */
  protected function outputs() {
    $outputs = array();
    for ($i = 0; $i < $this->outputCount; ++$i) {
      $outputs[] = 'output' . $i;
    }
    return $outputs;
  }

  /**
   * Implementation of TfOperation::outputInfo().
   */
  protected function outputInfo($outputKey, $propertyKey) {
    switch ($propertyKey) {
      case 'label':
        return t('$output[!num]', array('!num' => substr($outputKey, 6)));

      case 'expectedSchema':
        return new TfWritableSchemaElement($outputKey);
    }
  }

  /**
   * Implementation of TfOperation::execute().
   */
  protected function execute(TfOutput $_eval_op_output) {
    $input = array();
    for ($i = 0; $i < $this->inputCount; ++$i) {
      $input[$i] = $this->input('input' . $i);
      if ($input[$i]->isLeafElement()) {
        $input[$i] = $input[$i]->data();
      }
    }
    $output = array();
    eval($this->input('code')->data());

    for ($i = 0; $i < $this->outputCount; ++$i) {
      if (!array_key_exists($i, $output)) {
        $_eval_op_output->setErrorMessage(t('$output[!num] was not set by the evaluated PHP code.', array('!num' => $i)));
        return;
      }
      $_eval_op_output->set('output' . $i, $output[$i]);
    }
  }
}
