<?php
// $Id: operations.mapping.inc,v 1.3 2009/04/07 11:57:47 jpetso Exp $
/**
 * @file
 * Transformations -
 * A framework for generic schema-based data transformation pipelines.
 *
 * Copyright 2008 by Jakob Petsovits ("jpetso", http://drupal.org/user/56020)
 */

/**
 * Implementation of [module]_operation_[class]().
 */
function transformations_operation_TfSimpleMappingOutputsFromStructure() {
  return array(
    'label' => t('Extract child elements from structure (simple mapping)'),
    'description' => t('Provides child elements of an item structure as separate outputs. Translation is done by a simple source key to output name map.'),
  );
}

class TfSimpleMappingOutputsFromStructure extends TfOperation {
  private $map; // a cached, array-ified version of the 'map' input

  /**
   * Overriding TfOperation::initialize().
   */
  public function initialize(Array $typeOptions = array()) {
    $this->map = array();
    parent::initialize($typeOptions);
  }

  protected function initializeMap() {
    $this->map = array();

    if (!$this->isInputSet('map')) {
      return;
    }
    foreach ($this->input('map') as $sourceKey => $outputName) {
      $this->map[$sourceKey] = $outputName;
    }
  }

  /**
   * Called when input data has been (re-)set using setInput().
   */
  protected function inputChanged($inputKey) {
    parent::inputChanged($inputKey);

    if ($inputKey == 'map') {
      $this->initializeMap();
    }
  }

  /**
   * Implementation of TfOperation::inputs().
   */
  protected function inputs() {
    return array('map', 'structure');
  }

  /**
   * Implementation of TfOperation::inputInfo().
   */
  protected function inputInfo($inputKey, $propertyKey) {
    if ($inputKey == 'map') {
      switch ($propertyKey) {
        case 'label':
          return t('Key map');

        case 'description':
          return t('An association of source keys in the input structure (given as the keys of this "key map" list) to output names (given as the values of the key map).');

        case 'expectedSchema':
          $schema = new TfWritableSchemaElement($inputKey);
          $childElement = new TfWritableSchemaElement('targetKey', 'php:type:string');
          $childElement->setLabelRequirements(TfSchemaElement::NoMatchRequired);
          $cardinality = new TfWritableSchemaCardinality(
            $childElement, 1, TfSchemaCardinality::Unbounded
          );
          $collection = new TfSchemaSequence();
          $collection->addMember($cardinality);
          $schema->setChildCollection($collection);
          return $schema;
      }
    }
    elseif ($inputKey == 'structure') {
      switch ($propertyKey) {
        case 'label':
          return t('Item structure');

        case 'description':
          return t('The structure (list of items) whose child elements you want to extract.');

        case 'expectedSchema':
          $schema = new TfWritableSchemaElement($inputKey);
          $collection = new TfSchemaSetFragment();

          foreach ($this->map as $sourceKey => $outputName) {
            $collection->addMember(new TfWritableSchemaElement($outputName));
          }
          $schema->setChildCollection($collection);
          return $schema;
      }
    }
  }

  /**
   * Implementation of TfOperation::outputs().
   */
  protected function outputs() {
    return array_keys($this->map);
  }

  /**
   * Implementation of TfOperation::outputInfo().
   */
  protected function outputInfo($outputKey, $propertyKey) {
    switch ($propertyKey) {
      case 'label':
        return empty($this->map[$outputKey])
          ? t('(empty)')
          : $this->map[$outputKey];

      case 'expectedSchema':
        ///TODO: reuse original schema members if they are known
        return new TfWritableSchemaElement($outputKey);
    }
  }

  /**
   * Implementation of TfOperation::execute().
   */
  protected function execute(TfOutput $output) {
    foreach ($this->input('structure') as $sourceKey => $data) {
      if (isset($this->map[$sourceKey])) {
        $output->set($sourceKey, $data);
      }
    }
  }
}
